@@ -26,14 +26,18 @@ public class ReadCDF implements CDFConstants{
 
       jj = args.length;
       if (jj == 0) jj = 1;
-/*
+
       try {
+
+        System.out.println("CDF Library version: "+CDF.getLibraryVersion()+"\n");
+/*
         CDF.setValidate (VALIDATEFILEoff);
+*/
       } catch (CDFException vax) {
-        System.out.println("*** Error at setting the validation..."+vax);
+        System.out.println("*** Error... "+vax);
         System.exit(1);
       }
-*/
+
       for (ii = 0; ii < jj; ++ii) {
 	if (args.length != 0) fileName = args[ii];
         if (ii > 0) 
@@ -94,6 +98,11 @@ public class ReadCDF implements CDFConstants{
               System.out.println("Compression:  "+cdf.getCompression());
               if (cdf.getCompressionType() != 0)
                 System.out.println("cPct:         "+cdf.getCompressionPct());
+              if (new Integer(cdf.getVersion().substring(0,1)).intValue() > 3) {
+                long lsLastUpdated = cdf.getLeapSecondLastUpdated();
+                if (lsLastUpdated > 0)
+		  System.out.println("LeapSecondLastUpdated:     "+lsLastUpdated);
+              }
               System.out.println("Cache Size:   "+cdf.confirmCDFCacheSize());
   
               /**********************************************/
@@ -119,7 +128,7 @@ public class ReadCDF implements CDFConstants{
                        System.out.println (attrName+" ("+n+" entries):");
                 }
                 else {
-                   String currAttrName = a.getName();;
+                   String currAttrName = a.getName();
                    if (currAttrName != attrName) {
                        if (n <= 1)
                           System.out.println (currAttrName+" ("+n+" entry):");
@@ -130,30 +139,24 @@ public class ReadCDF implements CDFConstants{
                 i++;
                 Vector ent = a.getEntries();
                 for (Enumeration e1 =ent.elements() ; e1.hasMoreElements() ;) {
-                    Entry entry = (Entry) e1.nextElement();
-                    if (entry != null) {
-                          long eDataType = entry.getDataType();
-                          System.out.print ("\t"+entry.getID()+" ("+
+                   Entry entry = (Entry) e1.nextElement();
+                   if (entry != null) {
+                      long eDataType = entry.getDataType();
+                      System.out.print ("\t"+entry.getID()+" ("+
                                          CDFUtils.getStringDataType(eDataType)+
                                          "/"+entry.getNumElements()+
                                          "): \t");
-                          Object data = entry.getData();
-                          if (eDataType == CDF_EPOCH) {
-                              CDFUtils.printData (
-                                Epoch.encode(((Double) data).doubleValue()));
-                          } else if (eDataType == CDF_EPOCH16) {
-                              CDFUtils.printData (Epoch16.encode((double[])data));
-                          } else if (eDataType == CDF_TIME_TT2000) {
-                              CDFUtils.printData (
-                                CDFTT2000.encode(((Long)data).longValue()));
-                          } else {
-                            if (eDataType == CDF_CHAR || eDataType == CDF_UCHAR)
-                              System.out.print("\"");
-                            CDFUtils.printData (data);
-                            if (eDataType == CDF_CHAR || eDataType == CDF_UCHAR)
-                              System.out.print("\"");
-                          }
-                          System.out.println (" ");
+                      Object data = entry.getData();
+                      if (eDataType == CDF_EPOCH) {
+                         System.out.println(Epoch.toEncode(((Double) data).doubleValue()));
+                      } else if (eDataType == CDF_EPOCH16) {
+                         System.out.println(Epoch16.toEncode((double[])data));
+                      } else if (eDataType == CDF_TIME_TT2000) {
+                         System.out.println(CDFTT2000.toEncode(((Long)data).longValue()));
+                      } else {
+                         CDFUtils.printData (data);
+                      }
+//                       System.out.println (" ");
                     }
                 }
                 System.out.println (" ");
@@ -189,31 +192,49 @@ public class ReadCDF implements CDFConstants{
                        Variable v = cdf.getVariable(entry.getID());
                        System.out.print ("\t"+v.getName()+" ("+
                                         CDFUtils.getStringDataType(eDataType)+
-                                        "/"+entry.getNumElements()+
-                                        "): ");
-
+                                        "/"+entry.getNumElements()+"): ");
                        Object data = entry.getData();
                        if (eDataType == CDF_EPOCH) {
-                           CDFUtils.printData (
-                               Epoch.encode(((Double) data).doubleValue()));
+                          System.out.println(Epoch.toEncode(((Double) data).doubleValue()));
                        } else if (eDataType == CDF_EPOCH16) {
-                           CDFUtils.printData (Epoch16.encode((double[])data));
+                          System.out.println(Epoch16.toEncode((double[])data));
                        } else if (eDataType == CDF_TIME_TT2000) {
-                           CDFUtils.printData (
-                               CDFTT2000.encode(((Long) data).longValue()));
+                          System.out.println(CDFTT2000.toEncode(((Long) data).longValue()));
                        } else {
-                          if (eDataType == CDF_CHAR || eDataType == CDF_UCHAR)
-                            System.out.print("\"");
-                          CDFUtils.printData (data);
-                          if (eDataType == CDF_CHAR || eDataType == CDF_UCHAR)
-                            System.out.print("\"");
+                          if (entry.getScope() == VARIABLE_SCOPE) {
+                            if (CDFUtils.isStringDataType(eDataType)) {
+                              if (data instanceof String[]) {
+                                long ix = ((String[])data).length;
+                                System.out.println("\t("+ix+" string"+(ix>1?"s":"")+") ");
+                                int iy;
+                                for (iy=0; iy<ix; ++iy) {
+                                  if (iy != (ix-1))
+                                    System.out.println("\t\t\t\t\""+((String[])data)[iy]+"\""); 
+                                  else
+                                    System.out.print("\t\t\t\t\""+((String[])data)[iy]+"\"");
+                                }
+                              } else {
+                                CDFUtils.printData (data);
+                              }
+                            } else {
+                              String format;
+                              try {
+                                Attribute attr = cdf.getAttribute("FORMAT");
+                                Entry formatEntry = attr.getEntry (entry.getID());
+                                format = (String) formatEntry.getData();
+                              } catch (CDFException ee) {
+                                format = null;
+                              }
+                              CDFUtils.printDataWithFormat (data, format);
+                            }
+                          } else
+                            CDFUtils.printData (data);
                        }
                        System.out.println (" ");
                     }
                 }
                 System.out.println (" ");
               }
-
               /**************************************/
               /* Print out the Variable information */ 
               /**************************************/
@@ -226,7 +247,7 @@ public class ReadCDF implements CDFConstants{
               Vector  vars = cdf.getVariables();
 
               System.out.println("\nVariable Information ("+n+" variables)\n"+
-                                 "=========================================");
+                                 "===================================================");
               for (Enumeration e = vars.elements() ; e.hasMoreElements() ;) {
                 Variable v = (Variable) e.nextElement();
 
@@ -239,7 +260,7 @@ public class ReadCDF implements CDFConstants{
                 dt = v.getDataType();
                 dataType = CDFUtils.getStringDataType(dt);
                 dataType = dataType + "/" + String.valueOf(v.getNumElements());
-                noOfBlanks = 14 - dataType.length();
+                noOfBlanks = 20 - dataType.length();
                 for (i=0; i < noOfBlanks; i++)
                     dataType = dataType + " ";
 
@@ -249,7 +270,7 @@ public class ReadCDF implements CDFConstants{
                     if (i > 0) System.out.print (",");
                     System.out.print (dimSizes[i]); 
                 }
-                System.out.print ("]\t");
+                System.out.print ("]  ");
                 // if (numDims == 1) System.out.print ("\t");
 
                 System.out.print((v.getRecVariance() ? "T" : "F")+"/");
@@ -263,16 +284,35 @@ public class ReadCDF implements CDFConstants{
                      System.out.print (" sparseRecord=\""+sparseRecord+"\"");
 
                 // Print the user-define pad value if one exists. 
-		if (v.checkPadValueExistence()) {
+		// if (v.checkPadValueExistence()) {
+/*
                   padValue = v.getPadValue(); 
                   if (dt == CDF_EPOCH)
-                    padValue = Epoch.encode(((Double) padValue).doubleValue());
+                    padValue = Epoch.toEncode(((Double) padValue).doubleValue());
                   else if (dt == CDF_EPOCH16)
-                    padValue = Epoch16.encode((double[]) padValue);
+                    padValue = Epoch16.toEncode((double[]) padValue);
                   else if (dt == CDF_TIME_TT2000)
-                    padValue = CDFTT2000.encode(((Long) padValue).longValue());
-                  System.out.print (" PadValue=\""+padValue+"\"");
-                }
+                    padValue = CDFTT2000.toEncode(((Long) padValue).longValue());
+                  else if (dt != CDF_REAL4 && dt != CDF_FLOAT &&
+                           dt != CDF_REAL8 && dt != CDF_DOUBLE)
+                    System.out.print (" PadValue="+((CDFUtils.isStringDataType(dt)||
+                                                     CDFUtils.isEpochDataType(dt))?
+                                                    "\"":"")+padValue+
+                                                    ((CDFUtils.isStringDataType(dt)||
+                                                     CDFUtils.isEpochDataType(dt))?"\"":""));
+                  else {
+                      String format;
+                      try {
+                        Attribute attr = cdf.getAttribute("FORMAT");
+                        format = (String) attr.getEntry (v.getID()).getData();
+                      } catch (CDFException ee) {
+                        format = null;
+                      }
+                      System.out.print (" PadValue=");
+                      CDFUtils.printDataWithFormat (padValue,format);
+                  }
+*/
+                // }
 
                 blockingFactor = v.getBlockingFactor();
                 if (blockingFactor > 0)
@@ -308,6 +348,14 @@ public class ReadCDF implements CDFConstants{
                 numRecs = v.getNumWrittenRecords();
                 varName = v.getName();
                 System.out.println (varName);
+                String format;
+                try {
+                  Attribute attr = cdf.getAttribute("FORMAT");
+                  format = (String) attr.getEntry (v.getID()).getData();
+                } catch (CDFException ee) {
+                  format = null;
+                }
+
                 for (i=0; i < varName.length(); i++)
                      System.out.print ("-");
                 System.out.println (" ");
@@ -324,17 +372,20 @@ public class ReadCDF implements CDFConstants{
                   if (pv != null) {
 		    if (myDT == CDF_EPOCH)
 		      System.out.println ("Pad value:        "+
-                         Epoch.encode(((Double)pv).doubleValue()));
+                         Epoch.toEncode(((Double)pv).doubleValue()));
 		    else if (myDT == CDF_EPOCH16)
 		      System.out.println ("Pad value:        "+
-                         Epoch16.encode((double[])pv));
+                         Epoch16.toEncode((double[])pv));
                     else if (myDT == CDF_TIME_TT2000)
                       System.out.println ("Pad value:        "+
                          CDFTT2000.toUTCstring(((Long)pv).longValue()));
-                    else if (myDT == CDF_CHAR || myDT == CDF_UCHAR)
+                    else if (CDFUtils.isStringDataType(myDT))
                       System.out.println ("Pad value:        \""+ pv+"\"");
-                    else 
-		      System.out.println ("Pad value:        "+ pv);
+                    else {
+                      System.out.print ("Pad value:        ");
+                      CDFUtils.printDataWithFormat (pv, format);
+//                      System.out.println("");
+                    }
 		  }
 		}
                 System.out.println ("Records:          "+
@@ -366,7 +417,7 @@ public class ReadCDF implements CDFConstants{
                      status = cdf.getStatus();
                      if (status == CDF_OK || status != VIRTUAL_RECORD_DATA) {
                        System.out.println ("Record # "+i+":");
-                       data.dumpData();
+                       data.dumpDataWithFormat(format);
                      }
                 }
                 System.out.println("");
