@@ -41,7 +41,7 @@
 #define DELETEv64(fp,vStats) (V_delete64(fp,vStats) == 0)
 #define CACHEv64(fp,nBuffers) (V_setcache64(fp,nBuffers) == 0)
 #define FLUSHv64(fp) (V_flush64(fp) == 0)
-
+#define FREECACHESv64(fp) (V_freecaches64(fp))
 /******************************************************************************
 * LoadCVVRx/LoadVVRx.
 ******************************************************************************/
@@ -487,7 +487,7 @@ struct CVVRstruct64 {
   Int32 RecordType;     /* Type of record (ie. CVVR) */
   Int32 rfuA;           /* Reserved for future use.  Set to zero (0). */
   OFF_T cSize;          /* Size in bytes of compressed data. */
-			/* Bytes... */
+  void  *data;		/* Bytes... */
 			/* Extra... */
 };
 
@@ -495,6 +495,7 @@ struct CVVRstruct64 {
 #define CVVR_RECORDTYPE_OFFSET64 8
 #define CVVR_RFUa_OFFSET64      12
 #define CVVR_CSIZE_OFFSET64     16
+#define CVVR_DATA_OFFSET64      24
 
 /******************************************************************************
 * UIR structure/fields/offsets - Unused Internal Record.
@@ -732,7 +733,7 @@ STATICforIDL CDFstatus DeleteCDFfiles64 PROTOARGs((struct CDFstruct *CDF));
 STATICforIDL CDFstatus DeleteEntry64 PROTOARGs((
   struct CDFstruct *CDF, OFF_T aOffset, OFF_T eOffset
 ));
-STATICforIDL CDFstatus DecompressCDF64 PROTOARGs((vFILE *dotFp, vFILE *uDotFp));
+STATICforIDL CDFstatus DecompressCDF64 PROTOARGs((struct CDFstruct *CDF, vFILE *dotFp, vFILE *uDotFp));
 STATICforIDL CDFstatus WriteCompressedCDF64 PROTOARGs((
   struct CDFstruct *CDF, struct CPRstruct64 *CPR, Logical empty
 ));
@@ -815,7 +816,7 @@ STATICforIDL CDFstatus VariableType64 PROTOARGs((
   struct CDFstruct *CDF, OFF_T vdrOffset, Logical zVar, int *vType
 ));
 STATICforIDL CDFstatus InitVarStage64 PROTOARGs((
-  struct CDFstruct *CDF, struct VarStruct *Var, Int32 nBytes
+  struct CDFstruct *CDF, struct VarStruct *Var, OFF_T nBytes
 ));
 STATICforIDL CDFstatus CopyBytes64 PROTOARGs((
   vFILE *iFp, OFF_T iStart, CDFstatus iError, OFF_T nBytes, vFILE *oFp,
@@ -845,11 +846,12 @@ STATICforIDL CDFstatus UpdateVXRtailInVDR64 PROTOARGs((
 ));
 STATICforIDL CDFstatus UpdateConversions64 PROTOARGs((struct CDFstruct *CDF));
 STATICforIDL CDFstatus Compress64 PROTOARGs((
-  vFILE *iFp, OFF_T iOffset, OFF_T iSize, CDFstatus iError, Int32 cType,
-  Int32 cParms[], vFILE *oFp, OFF_T oOffset, OFF_T *oSize, CDFstatus oError
+  struct CDFstruct *CDF, vFILE *iFp, OFF_T iOffset, OFF_T iSize, 
+  CDFstatus iError, Int32 cType, Int32 cParms[], vFILE *oFp, OFF_T oOffset,
+  OFF_T *oSize, CDFstatus oError
 ));
 STATICforIDL CDFstatus Decompress64 PROTOARGs((
-  vFILE *iFp, OFF_T iOffset, OFF_T iSize, CDFstatus iError, Int32 cType,
+  struct CDFstruct *CDF, vFILE *iFp, OFF_T iOffset, OFF_T iSize, CDFstatus iError, Int32 cType,
   Int32 cParms[], vFILE *oFp, OFF_T oOffset, CDFstatus oError
 ));
 STATICforIDL CDFstatus DecompressToStage64 PROTOARGs((
@@ -873,6 +875,12 @@ VISIBLE_PREFIX Logical Read64_64 PROTOARGs((vFILE *fp, OFF_T *value));
 #if !defined(__CFM68K__) || defined(__USING_STATIC_LIBS__) || !defined(CFM68KDLL)
   VISIBLE_PREFIX CDFstatus UpdateDotCDF64 PROTOARGs((struct CDFstruct *CDF));
 #endif
+VISIBLE_PREFIX CDFstatus FlushAllVarStages64 PROTOARGs((
+  struct CDFstruct *CDF, struct VarStruct *Var
+));
+VISIBLE_PREFIX Logical CheckForOverCaches64 PROTOARGs((
+  struct CDFstruct *CDF, struct VarStruct *Var
+));
 VISIBLE_PREFIX vFILE *V_open64 PROTOARGs((char *file_spec, char *a_mode));
 VISIBLE_PREFIX int V_setcache64 PROTOARGs((vFILE *vfp, int nCacheBuffers));
 VISIBLE_PREFIX int V_seek64 PROTOARGs((vFILE *vfp, OFF_T offset, int direction));
@@ -889,6 +897,7 @@ VISIBLE_PREFIX int V_flush64 PROTOARGs((vFILE *vfp));
 VISIBLE_PREFIX int V_close64 PROTOARGs((vFILE *vfp, struct CDFstruct *CDF, 
   vSTATS *vStats));
 VISIBLE_PREFIX int V_delete64 PROTOARGs((vFILE *vFp, vSTATS *vStats));
+VISIBLE_PREFIX void V_freecaches64 PROTOARGs((vFILE *vFp));
 VISIBLE_PREFIX Logical FlushCache64 PROTOARGs((vFILE *vFp, 
   vCACHE *firstCache));
 VISIBLE_PREFIX CDFstatus ReadVXR64 VARPROTOARGs((vFILE *fp, OFF_T offset, ...));
