@@ -74,6 +74,7 @@
 #define EPOCHRANGEqual          22
 #define RECORDRANGEqual         23
 #define ZISO8601qual            24
+#define NOVALIDATEqual          25
 
 #define FORMATopt               0
 #define NOFORMATopt             1
@@ -815,6 +816,7 @@ char ***varsInGroup;
 struct ItemStruct ***varPtrsInGroup;
 int *numVarsInGroup;
 Logical ziso8601;
+Logical toValidate;
 #else
 extern int batchMode;
 extern Logical simpleMode;
@@ -872,6 +874,7 @@ extern char ***varsInGroup;
 extern struct ItemStruct ***varPtrsInGroup;
 extern int *numVarsInGroup;
 extern Logical ziso8601;
+extern Logical toValidate;
 #endif
 
 /******************************************************************************
