@@ -194,13 +194,6 @@ switch (Va->item) {
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
-/*
-    if (!sX(LocateCurrentVar64(CDF,zOp,&offset,&zVar,NULL),&pStatus)) {
-      AbortAccess64 (CDF, UPDATE, noDELETE);
-      return pStatus;
-    }
-*/
-
     zVar = CurrentVarMode(CDF,zOp);
     if (zModeON(CDF) || zVar) offset = CDF->CURzVarOffset64;
     else offset = CDF->CURrVarOffset64;
