@@ -129,13 +129,6 @@ switch (Va->item) {
     if (!CDF->singleFile) return UNSUPPORTED_OPERATION;
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
-/*
-    if (!sX(LocateCurrentVar(CDF,zOp,&vOffset,&zVar,NULL),&pStatus)) {
-      AbortAccess (CDF, UPDATE, noDELETE);
-      return pStatus;
-    }
-*/
-
     zVar = CurrentVarMode(CDF,zOp);
     if (zModeON(CDF) || zVar) vOffset = CDF->CURzVarOffset;
     else vOffset = CDF->CURrVarOffset;
@@ -307,6 +300,8 @@ switch (Va->item) {
       CDF->CURrVarNum = RESERVED_VARNUM;
       CDF->CURrVarOffset = 0;
     }
+    CDF->keptVarzOp = zOp;
+    CDF->keptVarNum = -1;
     /**************************************************************************
     * Delete the associated attribute entries and renumber the entries that
     * are associated with the variables that followed the variable that was
@@ -743,7 +738,7 @@ Logical reNumber;
     * Standard variable in a single-file CDF.
     **************************************************************************/
     case STANDARD_: {
-      Int32 vxrHead, aboveRecord, deleteCount; Int32 adjust;
+      Int32 vxrHead, aboveRecord, deleteCount;
       /************************************************************************
       * Verify/adjust the records to be deleted.
       ************************************************************************/
@@ -803,7 +798,7 @@ Logical reNumber;
     * Sparse records.
     **************************************************************************/
     case SPARSE_RECORDS_: {
-      Int32 aboveRecord, deleteCount;  Int32 adjust;
+      Int32 aboveRecord, deleteCount;
       Int32 vxrHead; Int32 maxRec;
       /************************************************************************
       * Flush staging area first.  This will automatically clear the staging
@@ -912,7 +907,7 @@ Logical reNumber;
     * Compressed records.
     **************************************************************************/
     case COMPRESSED_: {
-      Int32 vxrHead, aboveRecord, deleteCount; Int32 adjust;
+      Int32 vxrHead, aboveRecord, deleteCount;
       /************************************************************************
       * Flush/clear staging area first.
       ************************************************************************/
@@ -1332,7 +1327,7 @@ Int32 irSize;
       /************************************************************************
       * Compress the block of remaining records...
       ************************************************************************/
-      if (!sX(Compress(CDF->stage.fp,offset2,
+      if (!sX(Compress(CDF,CDF->stage.fp,offset2,
 		       nBytes2,SCRATCH_READ_ERROR,
 		       Var->cType,Var->cParms,
 		       CDF->compressFp,ZERO_OFFSET,
@@ -1534,7 +1529,7 @@ Int32 irSize;
       /************************************************************************
       * Compress the first block of remaining records...
       ************************************************************************/
-      if (!sX(Compress(CDF->stage.fp,
+      if (!sX(Compress(CDF,CDF->stage.fp,
 		       Var->stage.areaOffset,
 		       nBytes1,SCRATCH_READ_ERROR,
 		       Var->cType,Var->cParms,
@@ -1557,7 +1552,7 @@ Int32 irSize;
       /************************************************************************
       * Compress the second block of remaining records...
       ************************************************************************/
-      if (!sX(Compress(CDF->stage.fp,offset2,
+      if (!sX(Compress(CDF,CDF->stage.fp,offset2,
 		       nBytes2,SCRATCH_READ_ERROR,
 		       Var->cType,Var->cParms,
 		       CDF->compressFp,ZERO_OFFSET,
@@ -1663,7 +1658,7 @@ Int32 irSize;
       /************************************************************************
       * Compress the block of remaining records...
       ************************************************************************/
-      if (!sX(Compress(CDF->stage.fp,
+      if (!sX(Compress(CDF,CDF->stage.fp,
 		       Var->stage.areaOffset,
 		       nBytes1,SCRATCH_READ_ERROR,
 		       Var->cType,Var->cParms,
