@@ -206,6 +206,14 @@ switch (Va->item) {
       case NO_COMPRESSION:
 	/**********************************************************************
 	* If the CDF is currently compressed...
+	*   -- dotFp is compressed
+	*   -- uDotFp is uncompressed (from dotFp)
+	*   1. delete dotFp
+	*   2. re-create dotFp
+	*   3. set dotFp's cache size
+	*   4. copy uDotFp to dotFp
+	*   5. flush dotFp
+	*   6. delete uDotFp
 	**********************************************************************/
 	if (CDF->uDotFp != NULL) {
 	  char pathName[DU_MAX_PATH_LEN+1];
@@ -288,8 +296,19 @@ switch (Va->item) {
 	**********************************************************************/
 	if (!WriteAccess64(CDF,FALSE,&pStatus)) return pStatus;
 	/**********************************************************************
-	* If the CDF is not currently compressed, create an uncompressed dotCDF
+	* If the CDF is not currently compressed (uDotfp is NULL otherwise
+	* cdfope will create one), create an uDotfp
 	* file and copy the current contents into it.
+	*   -- dotFp is not compressed (as uDotFp does not exist)
+	*   -- uncompressed CDF to compressed CDF --
+	*   1. make uDotFp as a scratch
+	*   2. set uDotFp's cache size
+	*   3. copy dotFp to uDotFp
+	*   4. flush uDotFp --- creating a temporary file (actully a CDF - old)
+	*   5. delete dotFp
+	*   6. recreate dotFp
+	*   7. write compressed CDF to dotFp
+	*   8. flush dotFp 
 	**********************************************************************/
 	if (CDF->uDotFp == NULL) {
 	  CDF->uDotFp = V_scratch (ScratchDirectory(CDF), "cdf");
@@ -297,6 +316,7 @@ switch (Va->item) {
 	    AbortAccess64 (CDF, UPDATE, noDELETE);
 	    return CDF_CREATE_ERROR;
 	  }
+	  CDF->workingCacheSize = NUMcacheCOMPRESS;
 	  if (!CACHEv64(CDF->uDotFp,CDF->workingCacheSize)) {
 	    AbortAccess64 (CDF, UPDATE, noDELETE);
 	    return BAD_CACHE_SIZE;
@@ -407,7 +427,6 @@ switch (Va->item) {
     else
         Var = BOO(zOp,CDF->zVars[(int)CDF->CURzVarNum],
                       CDF->rVars[(int)CDF->CURrVarNum]);
-
     /**************************************************************************
     * Read VDR.
     **************************************************************************/
@@ -434,7 +453,7 @@ switch (Va->item) {
       if (cType != NO_COMPRESSION) return CANNOT_COMPRESS;
     }
     /**************************************************************************
-    * If the existing blocking factor is too small, then give warning.
+    * If the existing blocking factor is too small/large, then give warning.
     **************************************************************************/
     if (!zOp) {
       if (!sX(ReadGDR64(CDF->fp,CDF->GDRoffset64,
@@ -446,14 +465,32 @@ switch (Va->item) {
       }
     }
     if (RECvaryBITset(VDR.Flags) && (VDR.blockingFactor != 0)) {
+      OFF_T total;
       recSize = VDR.NumElems;
       recSize *= (Int32) CDFelemSize(VDR.DataType);
       for (i = 0; i < (int) (zOp?VDR.zNumDims:numDims); ++i) 
 	if (VDR.DimVarys[i]) recSize *= (zOp?VDR.zDimSizes[i]:dimSizes[i]);
-      if (VDR.blockingFactor < 
-	  (Int32) (MIN_BLOCKING_BYTES_compressed/recSize)) {
-	sX (BLOCKINGFACTOR_TOO_SMALL, &pStatus);
-/*	return pStatus; */
+      total = (OFF_T) recSize * VDR.blockingFactor;
+      if (total < (long) MIN_BLOCKING_BYTES_compressed) {
+      }
+      if (total > (OFF_T) NUMcacheSTAGE * MIN_BLOCKING_BYTES_compressed) {
+	int numCaches, iy; Int32 blocking;
+	numCaches = (int) ((total-1) / MIN_BLOCKING_BYTES_compressed) + 1;
+	blocking = VDR.blockingFactor;
+	iy = 0;
+	while (numCaches > NUMcacheSTAGE) {
+	  blocking = blocking / 2;
+	  if (blocking < 0) {
+	    blocking = 1;
+	    iy = 1;
+	    break;
+	  }	
+	  total = (OFF_T) recSize * blocking;
+	  numCaches = (int) ((total-1) / MIN_BLOCKING_BYTES_compressed) + 1;
+	}
+	if (iy == 0)
+	  sX (BLOCKINGFACTOR_TOO_LARGE, &pStatus);
+	VDR.blockingFactor = blocking;
       }
     }
     /**************************************************************************
