@@ -240,11 +240,31 @@ switch (Va->item) {
   case CDF_CACHESIZE_: {
     struct CDFstruct *CDF;
     int nBuffers = (int) va_arg (Va->ap, long);
+    int num;
     if (nBuffers < 0) return BAD_CACHE_SIZE;
     SelectCDF (Cur->cdf, CDF)
-    CDF->workingCacheSize = BOO(nBuffers > 0,nBuffers,BOO(CDF->singleFile,
-							  NUMcacheSINGLE,
-							  NUMcacheMULTI));
+    if (nBuffers == 0) {
+      if (CDF->singleFile) 
+	num = BOO(CDF->workingCacheSize > NUMcacheSINGLE,
+		  CDF->workingCacheSize, NUMcacheSINGLE);
+      else
+	num = BOO(CDF->workingCacheSize > NUMcacheMULTI,
+		  CDF->workingCacheSize, NUMcacheMULTI);
+    } else {
+/*
+      int num2;
+      if (CDF->singleFile) 
+	num2 =  BOO (nBuffers > NUMcacheSINGLE, nBuffers, NUMcacheSINGLE);
+      else
+	num2 =  BOO (nBuffers > NUMcacheMULTI, nBuffers, NUMcacheMULTI);
+      num = BOO(CDF->workingCacheSize > num2, CDF->workingCacheSize, num2);
+*/
+      num = BOO(CDF->workingCacheSize==NUMcacheCOMPRESS,
+                BOO(nBuffers>CDF->workingCacheSize,nBuffers,
+                                                   CDF->workingCacheSize),
+                nBuffers);
+    }
+    CDF->workingCacheSize = num;
     if (!CACHEv64(CDF->fp,CDF->workingCacheSize)) {
       AbortAccess64 (CDF, UPDATE, noDELETE);
       return BAD_CACHE_SIZE;
@@ -501,21 +521,29 @@ switch (Va->item) {
       return pStatus;
     }
     if (zModeON(CDF)) {
-      CDF->CURzVarNum = (zVar ? CDF->NrVars + varN : varN);
-      CDF->CURzVarOffset64 = offset;
+      if (CDF->CURzVarNum == (zVar ? CDF->NrVars + varN : varN)) {
+        CDF->CURzVarOffset64 = offset;
+      } else {
+        CDF->CURzVarNum = (zVar ? CDF->NrVars + varN : varN);
+        CDF->CURzVarOffset64 = offset;
+      }
     } else {
       if (zOp)
 	if (zVar) {
-	  CDF->CURzVarNum = varN;
-	  CDF->CURzVarOffset64 = offset;
+	  if (CDF->CURzVarNum != varN) {
+	    CDF->CURzVarNum = varN;
+	    CDF->CURzVarOffset64 = offset;
+	  }
 	} else
 	  return NO_SUCH_VAR;                   /* Wrong type of variable. */
       else
 	if (zVar)
 	  return NO_SUCH_VAR;                   /* Wrong type of variable. */
 	else {
-	  CDF->CURrVarNum = varN;
-	  CDF->CURrVarOffset64 = offset;
+	  if (CDF->CURrVarNum != varN) {
+	    CDF->CURrVarNum = varN;
+	    CDF->CURrVarOffset64 = offset;
+	  }
 	}
     }
     break;
