@@ -61,6 +61,8 @@
 #  include "errno.h"
 #endif
 
+static int ccc1 = 0, ccc11 = 0, ccc12 = 0, ccc2 = 0, ccc3 = 0;
+static int sc = 0, nsc = 0;
 /******************************************************************************
 * Local macros/typedef's.
 ******************************************************************************/
@@ -101,7 +103,7 @@ static Logical vRead PROTOARGs((
 static Logical vWrite PROTOARGs((
   long offset, void *buffer, size_t nBytes, vFILE *vFp
 ));
-static vCACHE *AllocateBuffer PROTOARGs((vFILE *vFp));
+static vCACHE *AllocateBuffer PROTOARGs((vFILE *vFp, Logical *new));
 static vCACHE *PageIn PROTOARGs((vFILE *vFp, long blockN));
 static Logical WriteBlockFromCache PROTOARGs((
   vFILE *vFp, vCACHE *cache, size_t Nbytes
@@ -141,28 +143,39 @@ static vCACHE *FindCache (vFp, blockN)
 vFILE *vFp;
 long blockN;
 {
-  vCACHE *cache = vFp->cacheHead;
-  while (cache != NULL) {
-    if (cache->blockN == blockN) {
-      if (cache != vFp->cacheHead) {
-	if (cache == vFp->cacheTail) {
-	  cache->prev->next = NULL;
-	  vFp->cacheTail = cache->prev;
+++ccc1;
+
+  if (vFp->scratch && vFp->fp == NULL && vFp->cacheNumInd != NULL) {
+++ccc11;
+    if (vFp->cacheNumInd[(int)blockN] == 1) {
+      return vFp->cacheLocPtr[(int)blockN];
+    } else
+      return NULL;
+  } else {
+    vCACHE *cache = vFp->cacheHead;
+    while (cache != NULL) {
+++ccc12;
+      if (cache->blockN == blockN) {
+	if (cache != vFp->cacheHead) {
+	  if (cache == vFp->cacheTail) {
+	    cache->prev->next = NULL;
+	    vFp->cacheTail = cache->prev;
+	  }
+	  else {
+	    cache->next->prev = cache->prev;
+	    cache->prev->next = cache->next;
+	  }
+	  vFp->cacheHead->prev = cache;
+	  cache->next = vFp->cacheHead;
+	  vFp->cacheHead = cache;
+	  cache->prev = NULL;
 	}
-	else {
-	  cache->next->prev = cache->prev;
-	  cache->prev->next = cache->next;
-	}
-	vFp->cacheHead->prev = cache;
-	cache->next = vFp->cacheHead;
-	vFp->cacheHead = cache;
-	cache->prev = NULL;
+	return cache;
       }
-      return cache;
+      cache = cache->next;
     }
-    cache = cache->next;
+    return NULL;
   }
-  return NULL;
 }
 
 /******************************************************************************
@@ -209,6 +222,7 @@ vCACHE *firstCache;     /* Pointer to the first cache structure to free. */
       cdf_FreeMemory (cache->ptr, NULL);
     cdf_FreeMemory (cache, NULL);
     cache = nextCache;
+ccc3++;
   }
   return TRUE;
 }
@@ -220,8 +234,9 @@ vCACHE *firstCache;     /* Pointer to the first cache structure to free. */
 * an error occurred).
 ******************************************************************************/
 
-static vCACHE *AllocateBuffer (vFp)
+static vCACHE *AllocateBuffer (vFp, new)
 vFILE *vFp;
+Logical *new;
 {
   vCACHE *cache; long nBytes;
 #if !defined(vms)
@@ -232,9 +247,13 @@ vFILE *vFp;
   * fail, process as if the maximum number of cache buffers has already been
   * reached.
   ****************************************************************************/
+  *new = FALSE;
   if (vFp->nBuffers < vFp->maxBuffers) {
     cache = (vCACHE *) cdf_AllocateMemory ((size_t)sizeof(vCACHE), NULL);
     if (cache != NULL) {
+  *new = TRUE;
+if (vFp->scratch) ++sc;
+else ++nsc;
 #if defined(MICROSOFTC_700) && INCLUDEvMEMORY
       if (useVmem)
 	cache->ptr = (void *) AllocateVMemory (nCACHE_BUFFER_BYTEs);
@@ -256,6 +275,7 @@ vFILE *vFp;
 	}
 	(vFp->nBuffers)++;
 	memset(cache->ptr, 0, nCACHE_BUFFER_BYTEs);
+ccc2++;
 	return cache;
       }
       else {
@@ -286,6 +306,7 @@ vFILE *vFp;
 	 vFp->cacheHead = cache;
 	 cache->prev = NULL;
        }
+ccc2++;
        return cache;
      }
   }
@@ -313,6 +334,7 @@ vFILE *vFp;
   }
   (vFp->nPageOuts)++;
   memset(cache->ptr, 0, nCACHE_BUFFER_BYTEs);
+ccc2++;
   return cache;
 }
 
@@ -393,8 +415,13 @@ vFILE *vFp;
 long blockN;
 {
   long offset, nBytes; vCACHE *cache; void *buffer;
-  cache = AllocateBuffer (vFp);
+  Logical new;
+  cache = AllocateBuffer (vFp, &new);
   if (cache == NULL) return NULL;
+  if (vFp->scratch && vFp->fp == NULL && new) {
+    vFp->cacheNumInd[blockN] = 1;
+    vFp->cacheLocPtr[blockN] = cache;
+  }
   offset = blockN * nCACHE_BUFFER_BYTEs;
   nBytes = vFp->phyLength - offset;
   nBytes = MINIMUM (nBytes, nCACHE_BUFFER_BYTEs);
@@ -617,8 +644,9 @@ int ix;
           strcatX (tmpPath, TmpFileTemplate, 0);
         }
         fileid = mkstemp(tmpPath);
-        if (fileid != -1) fp = fdopen(fileid, "w+");
-        else {
+        if (fileid != -1) {
+	  fp = fdopen(fileid, "w+");
+        } else {
           for (i = 1; i <= MAX_TMP; i++) {
             strcpyX (tmpPath, tmpFolder, (size_t)pathLength);
             snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
@@ -661,7 +689,9 @@ int ix;
 #       else
           snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
                     "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
-          if (!IsReg(tmpPath)) fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+          if (!IsReg(tmpPath)) {
+	    fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+	  }
 #       endif
 #     endif
     }
@@ -799,6 +829,8 @@ char *a_mode;           /* Access mode. */
   vFp->GDR64 = NULL;
   vFp->ADRList = NULL;
   vFp->ADRList64 = NULL;
+vFp->cacheNumInd = NULL;
+vFp->cacheLocPtr = NULL;
   /****************************************************************************
   * Determine length of file and set current offset.
   ****************************************************************************/
@@ -891,12 +923,61 @@ char *extension;	/* Extension to use for the scratch file.  If NULL,
   vFp->GDR64 = NULL;
   vFp->ADRList = NULL;
   vFp->ADRList64 = NULL;
+  vFp->cacheNumInd = (long *) cdf_AllocateMemory ((size_t)sizeof(long) *
+						  NUMcacheCOMPRESS, NULL);
+  vFp->cacheLocPtr = (vCACHE **) cdf_AllocateMemory ((size_t)sizeof(vCACHE *) *
+						     NUMcacheCOMPRESS, NULL);
+  memset (vFp->cacheNumInd, 0, NUMcacheCOMPRESS * sizeof(long));
+  memset (vFp->cacheLocPtr, 0, NUMcacheCOMPRESS * sizeof(vCACHE *));
   /****************************************************************************
   * Return pointer to vFILE structure.
   ****************************************************************************/
   return vFp;
 }
 
+/******************************************************************************
+* V_stagereset.
+* Reset the staging area and its scratch file pointer.
+******************************************************************************/
+
+VISIBLE_PREFIX void V_stagereset (CDF)
+struct CDFstruct *CDF;
+{
+  vFILE *vFp = CDF->stage.fp;
+  CDF->stage.mark64 = (OFF_T) ZERO_OFFSET64;
+  CDF->stage.mark = ZERO_OFFSET;
+  vFp->fp = NULL;
+  vFp->fh = 0;
+  vFp->scratch = TRUE;
+  vFp->error = FALSE;
+  vFp->eof = FALSE;
+  vFp->cacheHead = NULL;
+  vFp->cacheTail = NULL;
+  vFp->nBuffers = 0;
+  vFp->nBlockReads = 0;
+  vFp->nBlockWrites = 0;
+  vFp->nV_reads = 0;
+  vFp->nV_writes = 0;
+  vFp->nPageIns = 0;
+  vFp->nPageOuts = 0;
+  vFp->length = 0;
+  vFp->length64 = (OFF_T) 0;
+  vFp->phyLength = 0;
+  vFp->phyLength64 = (OFF_T) 0;
+  vFp->offset = 0;
+  vFp->offset64 = (OFF_T) 0;
+  vFp->GDR = NULL;
+  vFp->GDR64 = NULL;
+  vFp->ADRList = NULL;
+  vFp->ADRList64 = NULL;
+  vFp->cacheNumInd = (long *) cdf_AllocateMemory ((size_t)sizeof(long) *
+						  NUMcacheCOMPRESS, NULL);
+  vFp->cacheLocPtr = (vCACHE **) cdf_AllocateMemory ((size_t)sizeof(vCACHE *) *
+						     NUMcacheCOMPRESS, NULL);
+  memset (vFp->cacheNumInd, 0, NUMcacheCOMPRESS * sizeof(long));
+  memset (vFp->cacheLocPtr, 0, NUMcacheCOMPRESS * sizeof(vCACHE *));
+}
+
 /******************************************************************************
 * V_setcache.
 * Set number of cache buffers.  This can be done at any time after the file
@@ -944,6 +1025,17 @@ int maxBuffers;         /* New maximum number of cache buffers. */
   return 0;
 }
 
+/******************************************************************************
+* V_freecaches.
+* Free the caches from head to tail.
+******************************************************************************/
+
+VISIBLE_PREFIX void V_freecaches (vFp)
+vFILE *vFp;
+{
+  FreeCache(vFp->cacheHead);
+}
+
 /******************************************************************************
 * V_seek.
 * Seek to a position in the file.
@@ -1164,6 +1256,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   vCACHE *cache;        /* Pointer to cache structure. */
   Byte1 *cBuffer;       /* Pointer to cache buffer. */
   size_t nBytesInBlock;	/* Number of bytes to the end of the block. */
+  Logical new;
   /****************************************************************************
   * Validate write.
   ****************************************************************************/
@@ -1200,11 +1293,15 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 	}
       }
       else {
-	cache = AllocateBuffer (vFp);
+	cache = AllocateBuffer (vFp, &new);
 	if (cache == NULL) {
 	  vFp->error = TRUE;
 	  return 0;
 	}
+	if (vFp->scratch && vFp->fp == NULL && new) {
+	  vFp->cacheNumInd[firstBlockN] = 1;
+	  vFp->cacheLocPtr[firstBlockN] = cache;
+	}
 	cache->blockN = firstBlockN;
 #if CLEAR_BYTES
 	cBuffer = CACHEbufferWRITEto (cache);
@@ -1268,11 +1365,15 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 	   }
 	 }
 	 else {
-	   cache = AllocateBuffer (vFp);
+	   cache = AllocateBuffer (vFp, &new);
 	   if (cache == NULL) {
 	     vFp->error = TRUE;
 	     return 0;
 	   }
+	   if (vFp->scratch && vFp->fp == NULL && new) {
+	     vFp->cacheNumInd[blockN] = 1;
+	     vFp->cacheLocPtr[blockN] = cache;
+	   }
 	   cache->blockN = blockN;
 #if CLEAR_BYTES
 	   cBuffer = CACHEbufferWRITEto (cache);
@@ -1443,6 +1544,8 @@ vSTATS *vStats;         /* Pointer to statistics structure. */
   * Deallocate cache and vFILE structure.
   ****************************************************************************/
   FreeCache (vFp->cacheHead);
+  if (vFp->cacheNumInd != NULL) cdf_FreeMemory (vFp->cacheNumInd, NULL);
+  if (vFp->cacheLocPtr != NULL) cdf_FreeMemory (vFp->cacheLocPtr, NULL);
   cdf_FreeMemory (vFp->path, NULL);
   cdf_FreeMemory (vFp, NULL);
   /****************************************************************************
@@ -1495,6 +1598,8 @@ vSTATS *vStats;         /* Pointer to statistics structure. */
   * Deallocate cache and vFILE structure.
   ****************************************************************************/
   FreeCache (vFp->cacheHead);
+if (vFp->cacheNumInd != NULL) cdf_FreeMemory (vFp->cacheNumInd, NULL);
+if (vFp->cacheLocPtr != NULL) cdf_FreeMemory (vFp->cacheLocPtr, NULL);
   cdf_FreeMemory (vFp->path, NULL);
   cdf_FreeMemory (vFp, NULL);
   /****************************************************************************
