@@ -700,7 +700,7 @@ Logical *changed;	/* Assumed initialized to FALSE by caller. */
 	   };
 	   static struct PromptWindowStruct PW = {
 	     "Enter GZIP compression parameters...", 0, 0, 80, 1, gzipHeader,
-	     MAX_GZIP_TOKEN_LEN, token, 1, gzipTrailer, gzipExit,
+	     MAX_GZIP_TOKEN_LEN+1, token, 1, gzipTrailer, gzipExit,
 	     REFRESHkey_FSI, SOLkey_FSI, EOLkey_FSI, INSERTorOVERkey_FSI
 	   };
 	   if (gzipFirst) {
