@@ -284,7 +284,7 @@ struct VarStructST *Var;
     if (Var->rangeCheckVar) {
       if (LTx(Var->value,Var->validmin,Var->dataTypeV,Var->numElemsV))
         Var->low++;
-      else
+      else {
         if (GTx(Var->value,Var->validmax,Var->dataTypeV,Var->numElemsV))
 	  Var->high++;
         else {
@@ -298,6 +298,7 @@ struct VarStructST *Var;
 	  if (STRINGdataType(Var->dataTypeV))
 	    *(((char *)Var->maxINrange)+(int)Var->numElemsV) = '\0';
         }
+      }
     }
     Var->minmaxInited = TRUE;
   }
@@ -332,7 +333,7 @@ struct VarStructST *Var;
     if (Var->rangeCheckVar) {
       if (LTx(Var->value,Var->validmin,Var->dataTypeV,Var->numElemsV))
         Var->low++;
-      else
+      else {
         if (GTx(Var->value,Var->validmax,Var->dataTypeV,Var->numElemsV))
 	  Var->high++;
         else
@@ -363,6 +364,7 @@ struct VarStructST *Var;
             if (STRINGdataType(Var->dataTypeV))
 	      *(((char *)Var->maxINrange)+(int)Var->numElemsV) = '\0'; 
 	  }
+      }
     }
   }
   return;
